import os
import sys
import math
import time
import json
import logging
import argparse
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import warnings

class GPlotter():
    def __init__(self, exp_dir='./'):
        self.exp_dir = exp_dir

    def getBinwidth(self, list1):
        rangei = max(list1) - min(list1)
        if rangei == 0:
            return 1
        #print( 'range:', rangei)
        logi = math.log10(rangei)
        #print( max(list1), min(list1), rangei, logi)
        binwidth = 10**(int(logi)-1)
        if binwidth * 10 + min(list1) > max(list1):
            binwidth = binwidth/10
        #print( 'binwidth:', binwidth)
        return binwidth

    def getBinAndIntervals(self, list1):
        binwidth = self.getBinwidth(list1)
        mini = math.floor(min(list1) / binwidth) * binwidth
        maxi = math.ceil(max(list1)  / binwidth) * binwidth
        #print( mini, maxi, binwidth)
        intervals = np.arange(mini, maxi, binwidth)#.tolist()
        #print( 'first 10 items in interval:', intervals[:10])
        return binwidth, intervals

    def plotline(self, x_list, y_list, my_list_name,  unit, x_label='time (secs)', xlim=None):

        #line plot
        plot_args = {
                        'ncols':1,
                        'suptitle':my_list_name,
                        'figsize':(18,5),
                        'saveas':self.exp_dir + '/{}_ts.pdf'.format(my_list_name),
                        'show':True
                    }
        lines_args = [
                        {'x':x_list, 'y':y_list, 'color':'blue', 'subtitle':'',
                        'x_label':x_label, 'y_label':unit,
                        'plot_type':'line', 'xlim':xlim}
                     ]
        #plotter = PlotWrapper()
        #plotter.plotSubplots(plot_args, lines_args)
        self.plotSubplots(plot_args, lines_args)

    def plotdistr(self, my_list, my_list_name, unit, xlim=None):
        #p bars
        if len(my_list) == 0:
            print( 'length of list is 0 - skipping')
            return
        
        binwidth = self.getBinwidth(my_list) if len(my_list) > 0 else 1
        plot_args = {
                        'ncols':1,
                        'suptitle':my_list_name,
                        'figsize':(10,6),
                        'saveas':self.exp_dir + '/{}_dist.pdf'.format(my_list_name),
                        'show':True
                    }
        lines_args = [
                        {'x':my_list, 'y':None, 'color':'blue', 'subtitle':'',
                         'x_label':unit, 'y_label':'frequency',
                         'plot_type':'distr_hist', 'width':binwidth, 
                         'binwidth':binwidth, 'xlim':xlim}
                     ]
        self.plotSubplots(plot_args, lines_args)

    def multiSubPlotsLines(self, list_sub_vals, main_suptitle, minor_suptitle):
        '''
        list_sub_vals = [array of dicts]
        list_title = 'connection 0 - packet rate'
        list_subtitle = 'source: src_ip, src_port'
        '''

        plot_args = {
                        'ncols':2,
                        'suptitle':main_suptitle,
                        'figsize':(18,6),
                        'saveas':self.exp_dir + '/{}_{}_ts.pdf'.format(minor_suptitle, main_suptitle),
                        'show':True
                    }
        lines_args = []
        distr_args = []

        for subplot_i in list_sub_vals:
            #print( '\n        ' + minor_suptitle + subplot_i['subtitle'])
            list_vals = subplot_i['y_list']
            #print( 'length of list is ', len(list_vals))
            #print( 'first 20 elements of list is ', list_vals[:20])
            #print( 'first 20 elements of list is ', [i for i in enumerate(list_vals[:20])])
            x_vals = subplot_i['x_list'] if 'x_list' in subplot_i else range(len(list_vals))
            xlim = subplot_i['x_lim'] if 'x_lim' in subplot_i else None
            lines_args.append({'x':x_vals, 'y':list_vals, 'color':'blue',
             'subtitle':minor_suptitle+'_'+subplot_i['subtitle'],
             'x_label':subplot_i['x_label'], 'y_label':subplot_i['unit'],
             'plot_type':'line', 'xlim':xlim})
        self.plotSubplots(plot_args, lines_args)


    def multiSubPlotsDistr(self, list_sub_vals, main_suptitle, minor_suptitle):
        '''
        list_sub_vals = [array of dicts]
        list_title = 'connection 0 - packet rate'
        list_subtitle = 'source: src_ip, src_port'
        '''
        #line plot
        plot_args = {
                        'ncols':2,
                        'suptitle':main_suptitle + ' frequency distribution',
                        'figsize':(18,6),
                        'saveas':self.exp_dir + '/{}_dist.pdf'.format(minor_suptitle, main_suptitle),
                        'show':True
                    }
        lines_args = []
        distr_args = []
        for subplot_i in list_sub_vals:
            #print( '\n        ' + minor_suptitle + subplot_i['subtitle'])
            list_vals = subplot_i['y_list']
            #print( 'length of list is ', len(list_vals))
            #print( 'first 20 elements of list is ', list_vals[:20])
            #print( 'first 20 elements of list is ', [i for i in enumerate(list_vals[:20])])
            xlim= subplot_i['x_lim'] if 'x_lim' in subplot_i else None
            #binwidth = 10**(len(str(max(list_vals)))-2) if len(list_vals) > 0 else 1
            binwidth = self.getBinwidth(list_vals) if len(list_vals) > 0 else 1
            #print( subplot_i['subtitle'], 'binwidth', binwidth)
            lines_args.append({'x':list_vals, 'y':None, 'color':'blue', 
                              'subtitle':minor_suptitle+'_'+subplot_i['subtitle'],
                             'x_label':subplot_i['unit'], 'y_label':'frequency',
                             'plot_type':'distr_hist', 'width':binwidth, 
                             'binwidth':binwidth, 'xlim':xlim})
        self.plotSubplots(plot_args, lines_args)
        

    def plotCDF(self, data1, data_name, unit=None):
        if len(data1) == 0:
            print( 'cannot make CDF :- length of data is 0')
            return
        data1 = [float(x) for x in data1]
        sorted1 = np.sort(data1)
        
        try:
            p1 = 1. * np.arange(len(data1))/(len(data1) - 1)
        except RuntimeWarning:
            print ('len_data_1 = ', len(data1))
            print ('p1 = ', p1)

        plt.figure(figsize=(10,6))
        plt.plot(sorted1, p1, color = 'b')
        plt.title(data_name + '_cdf')
        #print( data_name)
        fname = data_name
        #print( fname)
        plt.savefig(self.exp_dir + '/' + fname + '.pdf')
        #plt.show()
        plt.gcf().clear()


    def plot2CDFs(self, data1, data2, main_suptitle, minor_suptitle, subtitle1, subtitle2, unit=None):

        data1 = [float(x) for x in data1]
        sorted1 = np.sort(data1)

        try:
            p1 = 1. * np.arange(len(data1))/(len(data1) - 1)
        except RuntimeWarning:
            print ('len_data_1 = ', len(data1))
            print ('p1 = ', p1)

        data2 = [float(x) for x in data2]
        sorted2 = np.sort(data2)
        
        try:
            p2 = 1. * np.arange(len(data2))/(len(data2) - 1)
        except RuntimeWarning:
            print ('len_data_1 = ', len(data2))
            print ('p2 = ', p2)
        
        fig=plt.figure(figsize=(18,6))
        fig.subplots_adjust(hspace=.5)
        fig.suptitle(main_suptitle + '_cdf')
        
        ax1=fig.add_subplot(1,2,1)
        ax1.set_title(minor_suptitle + '_' + subtitle1)
        ax1.grid()
        if len(data1) == 0:
            print( 'cannot make CDF 1:- length of data is 0')
        else:
            ax1.plot(sorted1, p1, color = 'b')

        ax2=fig.add_subplot(1,2,2)
        ax2.set_title(minor_suptitle + '_' + subtitle2)
        ax2.grid()
        if len(data2) == 0:
            print( 'cannot make CDF 2:- length of data is 0')
        else:
            ax2.plot(sorted2, p2, color = 'b')

        #plt.title(main_suptitle + '_cdf')
        plt.savefig(self.exp_dir + '/' + minor_suptitle + '_' + main_suptitle + '_cdf.pdf')
        #plt.show()
        plt.gcf().clear()


    def plotSubplots(self, plot_args, lines_args):
        '''
        plots a series of lines on the same axis
        args:
            plot_args : A dictionary containing aguments for the entire plot.
                        The format is as given below.
                        dict({
                            'ncolumns' = 1
                            'suptitle' = 'global_title'
                            'figsize':(width, height)
                            'saveas' = 'file_name_to_save'
                            'show' = False
                        })
            lines_args: A list of dictionaries. Each dictionary containing
                        details for each individual line series to be plotted.
                        The format is as given below.
                        [
                            {'x':[], 'y':[], 'color':'blue', 'subtitle':'subtitle',
                            'x_label':'x_axis_label', y_label':'y_axis_label',
                            'plot_type'='line/distr_hist/bar', 'width':1, 
                            'binwidth':10, 'xlim':None},
                            ...
                            {'x':[], 'y':[], 'color':'blue', 'subtitle':'subtitle',
                            'x_label':'x_axis_label', y_label':'y_axis_label',
                            'plot_type'='line/distr_hist/bar', 'width':1, 
                            'binwidth':10},
                        ]
        '''
        numplots = len(lines_args)
        ncols = plot_args['ncols'] if 'ncols' in plot_args else 1
        suptitle = plot_args['suptitle'] if 'suptitle' in plot_args else ''
        figsize = plot_args['figsize'] if 'figsize' in plot_args else (18,6)
        nrows = int(math.ceil(float(numplots)/ncols))
        fig=plt.figure(figsize=figsize)
        fig.subplots_adjust(hspace=.5)
        fig.suptitle(suptitle)
        for i, line in enumerate(lines_args):
            #print( 'nrows,ncols,i+1 - ', nrows,ncols,i+1)
            
            ax=fig.add_subplot(nrows,ncols,i+1)
            marker = line['marker'] if 'marker' in line else '+'
            color = line['color'] if 'color' in line else 'blue'
            linewidth = line['linewidth'] if 'linewidth' in line else 0.6
            subtitle = line['subtitle'] if 'subtitle' in line else ''
            y_label = line['y_label'] if 'y_label' in line else ''
            x_label = line['x_label'] if 'x_label' in line else ''
            width = line['width'] if 'width' in line else 1
            binwidth = line['binwidth'] if 'binwidth' in line else 10
            ax.set_title(subtitle)
            ax.set_ylabel(y_label)
            ax.set_xlabel(x_label)
            ax.grid()
            if len(line['x']) == 0:
                print( 'length of {} for {} is 0, skipping'.format(suptitle, subtitle))
                continue
            #if 'Packet Sizes' in suptitle:
            #    if line['y']:
            #        line['ylim']=(min(line['y'])*0.9, max(line['y'])*1.1)
            #        print( 'ylim', line['ylim'])
            #        print( line['y'][:20])
            #    if line['x']:
            #        line['xlim']=(min(line['x'])*0.9, max(line['x'])*1.1)
            #        width = 100
            #        line['xlim']=(0, 2000)
            #        print( 'xlim', line['xlim'])
            #        print( line['x'][:20])
                              
            if 'plot_type' not in line or line['plot_type'] == 'line':
                ax.plot(line['x'], line['y'], marker=marker, color=color,
                        linewidth=linewidth, label=y_label)
            elif line['plot_type'] == 'scatter':
                ax.scatter(line['x'], line['y'], marker=marker, color=color,
                        label=y_label)
            elif line['plot_type'] == 'bar':
                ax.bar(np.array(line['x']), line['y'], color=color, width=width)
            elif line['plot_type'] == 'distr_hist':
                low = math.floor((min(line['x']) / binwidth) * binwidth)
                bins = np.arange(low, max(line['x']) + binwidth, binwidth)
                ax.hist(line['x'], color = color, edgecolor = 'black', bins = bins, width=width)
            #print( 'xlim:', ax.get_xlim(), 'ylim:', ax.get_ylim())
            if 'xlim' in line and line['xlim'] != None:
                ax.set_xlim(line['xlim'][0], line['xlim'][1])
            if 'ylim' in line and line['ylim'] != None:
                ax.set_ylim(line['ylim'][0], line['ylim'][1])
        if plot_args['saveas']:
            plt.savefig(plot_args['saveas'])
        #if plot_args['show']:
        #    plt.show()
        plt.gcf().clear()
